# Hello CodeBot REST API - Java Client

Copyright (c) MattCo.

*** Contains robot-generated code which shouldn't be modified. ***

Generated by CodeBot from Parallel Agile, Inc.

# Project Dependencies

Uses the generated HelloCodeBot Domain Model library. This should either be installed to your local Maven repo,
or published to your organization's private Maven or Artifactory server. See the HelloCodeBot README.md.

For JSON writing and parsing, the API client uses Google's GSON library. We chose this over Jackson because benchmarks have shown
GSON to be faster for serialization of small objects, so it's well suited to a microservice architecture
(conversely, Jackson is faster for very large objects).

For an HTTP client, the API client uses Unirest-java: http://unirest.io/java.html
Unirest provides a fluent interface on top of Apache HttpComponents, in addition to Json object mapping via GSON.


# Building the Hello Code Bot Java Client

The API client includes a Maven build file (pom.xml).

To build, run:

```
mvn install -Dmaven.repo.local=/home/{your username}/.m2/repository
```

Then either add the packaged jar file directly to your project dependencies, or deploy to your organisation's Artifactory
server.

To build both the domain classes and API client, run:

```
./BuildJavaClient.sh
```

Note: If you're running Linux under Windows using WSL but are running an IDE in Windows,
you'll also need to copy the 'domain classes' jar file to your Windows Maven repo:

```
cp -r ~/.m2/repository/mattco /c/Users/{your username}/.m2/repository/
```


# Testing the API Client

To get started straightaway, try running the examples in src/main/java/mattco.hello.api.examples.
If your API is hosted, the examples are set up to use the hosted API.
You can also run them against a local copy of the express.js API, if you've downloaded and run it,
on localhost:7000.
If you have the Paid license, you can find this API in the /Server folder.


# Running the API Client from your own code

If the API requests are routed through an authentication gateway or similar, you can add the required HTTP headers
(e.g. "Authorisation" header) via HelloCodeBotApi.customHttpHeaders. These headers are added to every request.

If you need more control over the API client, e.g. configure a proxy server or a client certificate store,
extend HelloCodeBotApi in your own code, and override the applyConfig() method.


# A note about IDs

Every domain class has an id field (which you don't need to define in the UML model). The API maps this to/from the
relevant primary key field in your back-end database of choice. So in MongoDB, id is turned into an `_id` ObjectId,
in Oracle it's turned into a primary key called `id`, etc.

An ID value is automatically allocated when an object is created, so you don't need to supply a unique ID. This is the recommended approach.

If you do wish to provide a custom ID, however, just make sure it conforms to the database's requirements, e.g.
24-digit hex string for Mongo.


# Logging

The API client uses SLF4J for all its logging. This allows you to configure logging at the application level,
and to switch in the logging framework of choice, e.g. Logback or the built-in java.util.logging API.

So that the tests generate loggable output, we've included a logback-classic dependency with "test" scope. However,
to see the logged output when running your application (and avoid the "Failed to load class org.slf4j.impl.StaticLoggerBinder" error message),
you'll need to include a similar dependency (with runtime or default scope) in your own project.
