import HelloCodeBotApi from "../HelloCodeBotApi.js"
import CreateManyResult from "../CreateManyResult.js"
import CreateResult from "../CreateResult.js"
import DeleteResult from "../DeleteResult.js"
import FindManyResult from "../FindManyResult.js"
import FindResult from "../FindResult.js"
import DataHelper from "../DataHelper.js"

const api = HelloCodeBotApi.clientForLocalhost()

describe('JWT Test', () => {
	test('Login Test', async () => {
		const loginResult = await api.login("hello", "hellohello")
		expect(loginResult).toBe(true)
	})
})

describe('Single User API Calls (Create -> Retrieve -> Update -> Delete)', () => {
    let user = DataHelper.NewUser()

    test('Create one', async () => {
        const createResult = await api.userApi.createOne(user.toJSON(true));
        expect(createResult.isOk()).toBe(true)
        expect(createResult.id).toBeTruthy()
        user["id"] = createResult.id
    });

    // Read the entity back by its database ID:
    test('Retrieve by DB ID', async () => {
        const findResult = await api.userApi.findOne(user["id"])
        expect(findResult.isOk()).toBe(true)
    });

    // Updating string attributes of the entity
    test('Update', async () => {
        user.Username = "new Username";
        user.Password = "new Password";
        user.Email = "new Email";
        const updateResult = await api.userApi.update(user)
        expect(updateResult.isOk()).toBe(true)
    });

    // Read the entity back after update:
    test('Retrieve and check updated values', async () => {
        const findResult = await api.userApi.findOne(user["id"])
        expect(findResult.entity).toEqual(
            expect.objectContaining({
                "id": user["id"],
                "Username": "new Username",
                "Password": "new Password",
                "Email": "new Email",
            }))
    });

    test('Update selective attributes', async () => {
        user.Username = "newer Username";
        user.Password = "newer Password";
        user.Email = "newer Email";
        const updateResult = await api.userApi.updateSelectiveAttributes(user["id"], {
            "Username": user["Username"],
            "Password": user["Password"],
            "Email": user["Email"],
        });
        expect(updateResult.isOk()).toBe(true)
    });

    // Read the entity back after selective attributes update:
    test('Retrieve and check updated values', async () => {
        const findResult = await api.userApi.findOne(user["id"])
        expect(findResult.entity).toEqual(
            expect.objectContaining({
                "id": user["id"],
                "Username": "newer Username",
                "Password": "newer Password",
                "Email": "newer Email",
            }))
    });

    // Finally, delete the entity:
    test('Delete entity', async () => {
        const deleteResult = await api.userApi.deleteOne(user["id"])
        expect(deleteResult.count).toBe(1)
    });
});

describe('Multiple User in API Calls (CreateMany -> Retrieve -> DeleteMany)', () => {
    let createManyResult

    // Create Multiple entities
    test('Create many', async () => {
        let users = DataHelper.NewUserList(3);
        createManyResult = await api.userApi.createMany(users);
        expect(createManyResult.isOk()).toBe(true)
        for(let i=0;i<createManyResult.ids.length;i++)
            expect(createManyResult.ids[i]).toBeTruthy()
    });

    // Read multiple entities back
    test('Retrieve all entities', async () => {
        const findManyResult = await api.userApi.findAll()
        expect(findManyResult.isOk()).toBe(true)
        expect(findManyResult.entities).toEqual(
            expect.arrayContaining([
                expect.objectContaining({ "id": createManyResult.ids[0] }),
                expect.objectContaining({ "id": createManyResult.ids[1] }),
                expect.objectContaining({ "id": createManyResult.ids[2] })
            ]))
    });

    // Finally, delete multiple entities:
    test('Delete all entities', async () => {
        const deleteResult = await api.userApi.deleteAll()
        expect(deleteResult.count).toBeGreaterThanOrEqual(3)
    });
});